from sphinxcontrib.runcmd.utils import Singleton


class TheSingleton(Singleton):
    pass


def test_singleton():
    a = TheSingleton()
    b = TheSingleton()

    assert a == b
    assert hash(a) == hash(b)



import csv
import re
import shlex
import subprocess
import sys

from docutils.parsers.rst import directives
from sphinx.directives import code

from .utils import Singleton

__version__ = "0.2.0"

# CONSTANTS
RE_SPLIT = re.compile(r"(?P<pattern>.*)(?<!\\)/(?P<replacement>.*)")


class CMDCache(Singleton):
    cache = {}

    def get(self, cmd):
        h = hash(cmd)
        if h in self.cache:
            return self.cache[h]
        else:
            result = run_command(cmd)
            self.cache[h] = result
            return result


def run_command(command):
    true_cmd = shlex.split(command)
    try:
        subp = subprocess.Popen(
            true_cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE
        )
    except Exception as e:
        out = ""
        err = e
    else:
        out, err = subp.communicate()
        encoding = sys.getfilesystemencoding()
        out = out.decode(encoding, "replace").rstrip()
        err = err.decode(encoding, "replace").rstrip()

    if err and err != "":
        print("Error in runcmd: {}".format(err))
        out = "{}\n{}".format(out, err)

    return out


class RunCmdDirective(code.CodeBlock):
    has_content = False
    final_argument_whitespace = False
    required_arguments = 1
    optional_arguments = 99

    option_spec = {
        # code.CodeBlock option_spec
        "linenos": directives.flag,
        "dedent": int,
        "lineno-start": int,
        "emphasize-lines": directives.unchanged_required,
        "caption": directives.unchanged_required,
        "class": directives.class_option,
        "name": directives.unchanged,
        # RunCmdDirective option_spec
        "syntax": directives.unchanged,
        "replace": directives.unchanged,
        "prompt": directives.flag,
        "dedent-output": int,
    }

    def run(self):
        # Grab a cache singleton instance
        cache = CMDCache()

        # Get the command output
        command = " ".join(self.arguments)
        output = cache.get(command)

        # Grab our custom commands
        syntax = self.options.get("syntax", "text")
        replace = self.options.get("replace", "")
        reader = csv.reader([replace], delimiter=",", escapechar="\\")
        prompt = "prompt" in self.options
        dedent_output = self.options.get("dedent-output", 0)

        # Dedent the output if required
        if dedent_output > 0:
            output = "\n".join([x[dedent_output:] for x in output.split("\n")])

        # Add the prompt to our output if required
        if prompt:
            output = "{}\n{}".format(command, output)

        # Do our "replace" syntax on the command output
        for items in reader:
            for regex in items:
                if regex != "":
                    match = RE_SPLIT.match(regex)
                    p = match.group("pattern")
                    # Let's unescape the escape chars here as we don't need them to be
                    # escaped in the replacement at this point
                    r = match.group("replacement").replace("\\", "")
                    output = re.sub(p, r, output)

        # Set up our arguments to run the CodeBlock parent run function
        self.arguments[0] = syntax
        self.content = [output]
        node = super(RunCmdDirective, self).run()

        return node


def setup(app):
    app.add_directive("runcmd", RunCmdDirective)

    return {"version": __version__}



from sphinx_testing import with_app


@with_app(buildername="html", srcdir="./tests/examples", copy_srcdir_to_tmpdir=True)
def sphinx_build(app, status, warning):
    app.build()
    with open(app.outdir + "/index.html", "r") as f:
        html = f.read()
    assert "python test.py -h" in html
    assert "No such file or directory" in html


def test_build():
    sphinx_build()



import codecs

from setuptools import find_packages, setup

TEST_DEPS = ["coverage", "pytest", "pytest-cov", "sphinx_testing"]

EXTRAS = {"test": TEST_DEPS}

setup(
    name="sphinxcontrib-runcmd",
    version="0.2.0",
    author="Fernando Chorney",
    author_email="fernando.chorney@invenia.ca",
    url="https://github.com/invenia/sphinxcontrib-runcmd",
    download_url="https://pypi.org/project/sphinxcontrib-runcmd",
    license="MIT",
    description='Sphinx "runcmd" extension',
    long_description=codecs.open("README.md", "r", "utf-8").read(),
    long_description_content_type="text/markdown",
    zip_safe=False,
    packages=find_packages(exclude=["tests"]),
    include_package_data=True,
    install_requires=["sphinx"],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.6",
        "Topic :: Documentation",
        "Topic :: Utilities",
    ],
    platforms="any",
    test_require=TEST_DEPS,
    extras_require=EXTRAS,
    namespace_packages=["sphinxcontrib"],
)
